.form {
  @apply flex flex-col gap-2 xl:gap-6;
}

.group {
  @apply flex flex-col gap-2 xl:gap-4;

  .group {
    @apply xl:gap-2;
  }

  &.checkbox {
    @apply flex flex-row;

    input {
      @apply ml-auto align-middle;
    }
  }
}

.input {
  @apply px-4 py-2 border-[1px] focus:outline-none focus:border-uclouvain-1;
}

.heading {
  @apply text-lg xl:text-xl border-b-2 py-2;
}
