$parameters: ("1", "2", "3", "4");

@each $parameter in $parameters {
  .grid-#{$parameter}-l {
    grid-template-columns: max-content repeat(#{$parameter}, 1fr);
  }
  .grid-#{$parameter}-r {
    grid-template-columns: repeat(#{$parameter}, 1fr) min-content;
  }
  .grid-#{$parameter} {
    grid-template-columns: repeat(#{$parameter}, 1fr);
  }
}
